@@ -14,6 +14,7 @@ from ..plugins import PLUGINS, get_tools_result_async, function_call_list, updat
 from ..utils.scripts import safe_get, async_generator_to_sync, parse_function_xml, parse_continuous_json, convert_functions_to_xml, remove_xml_tags_and_content
 from ..core.request import prepare_request_payload
 from ..core.response import fetch_response_stream, fetch_response
+from ..architext.architext import Messages, SystemMessage, UserMessage, AssistantMessage, ToolCalls, ToolResults, Texts, RoleMessage, Images, Files
 
 class APITimeoutError(Exception):
     """Custom exception for API timeout errors."""
@@ -37,10 +38,18 @@ class RateLimitError(Exception):
     """Custom exception for rate limit (429) errors."""
     pass
 
+class BadRequestError(Exception):
+    """Custom exception for bad request (400) errors."""
+    pass
+
 class HTTPError(Exception):
     """Custom exception for HTTP 500 errors."""
     pass
 
+class InputTokenCountExceededError(Exception):
+    """Custom exception for input token count exceeding the maximum."""
+    pass
+
 class ConfigurationError(Exception):
     """Custom exception for configuration errors."""
     pass
@@ -80,7 +89,6 @@ class chatgpt(BaseLLM):
         print_log: bool = False,
         tools: Optional[Union[list, str, Callable]] = [],
         function_call_max_loop: int = 3,
-        cut_history_by_function_name: str = "",
         cache_messages: list = None,
         logger: logging.Logger = None,
         check_done: bool = False,
@@ -89,20 +97,13 @@ class chatgpt(BaseLLM):
         Initialize Chatbot with API key (from https://platform.openai.com/account/api-keys)
         """
         super().__init__(api_key, engine, api_url, system_prompt, proxy, timeout, max_tokens, temperature, top_p, presence_penalty, frequency_penalty, reply_count, truncate_limit, use_plugins=use_plugins, print_log=print_log)
-        self.conversation: dict[str, list[dict]] = {
-            "default": [
-                {
-                    "role": "system",
-                    "content": self.system_prompt,
-                },
-            ],
+        self.conversation: dict[str, Messages] = {
+            "default": Messages(SystemMessage(self.system_prompt)),
         }
         if cache_messages:
             self.conversation["default"] = cache_messages
         self.function_calls_counter = {}
         self.function_call_max_loop = function_call_max_loop
-        self.cut_history_by_function_name = cut_history_by_function_name
-        self.latest_file_content = {}
         self.check_done = check_done
 
         if logger:
@@ -156,91 +157,48 @@ class chatgpt(BaseLLM):
         if convo_id not in self.conversation:
             self.reset(convo_id=convo_id)
         if function_name == "" and message:
-            self.conversation[convo_id].append({"role": role, "content": message})
+            self.conversation[convo_id].append(RoleMessage(role, message))
         elif function_name != "" and message:
-            # 删除从 cut_history_by_function_name 以后的所有历史记录
-            if function_name == self.cut_history_by_function_name:
-                matching_message = next(filter(lambda x: safe_get(x, "tool_calls", 0, "function", "name", default="") == 'get_next_pdf', self.conversation[convo_id]), None)
-                if matching_message is not None:
-                    self.conversation[convo_id] = self.conversation[convo_id][:self.conversation[convo_id].index(matching_message)]
-
             if not (all(value == False for value in self.plugins.values()) or self.use_plugins == False):
-                self.conversation[convo_id].append({
-                    "role": "assistant",
-                    "tool_calls": [
-                        {
-                            "id": function_call_id,
-                            "type": "function",
-                            "function": {
-                                "name": function_name,
-                                "arguments": function_arguments,
-                            },
-                        }
-                    ],
-                    })
-                self.conversation[convo_id].append({"role": role, "tool_call_id": function_call_id, "content": message})
+                tool_calls = [
+                    {
+                        "id": function_call_id,
+                        "type": "function",
+                        "function": {
+                            "name": function_name,
+                            "arguments": function_arguments,
+                        },
+                    }
+                ]
+                self.conversation[convo_id].append(ToolCalls(tool_calls))
+                self.conversation[convo_id].append(ToolResults(tool_call_id=function_call_id, content=message))
             else:
                 last_user_message = self.conversation[convo_id][-1]["content"]
                 if last_user_message != message:
-                    image_message_list = []
+                    image_message_list = UserMessage()
                     if isinstance(function_arguments, str):
                         functions_list = json.loads(function_arguments)
                     else:
                         functions_list = function_arguments
                     for tool_info in functions_list:
                         if tool_info.get("base64_image"):
-                            image_message_list.append({"type": "text", "text": safe_get(tool_info, "parameter", "image_path", default="") + " image:"})
-                            image_message_list.append({
-                                "type": "image_url",
-                                "image_url": {
-                                    "url": tool_info["base64_image"],
-                                }
-                            })
-                    self.conversation[convo_id].append({"role": "assistant", "content": convert_functions_to_xml(function_arguments)})
+                            image_message_list.extend([
+                                safe_get(tool_info, "parameter", "image_path", default="") + " image:",
+                                Images(tool_info["base64_image"]),
+                            ])
+                    self.conversation[convo_id].append(AssistantMessage(convert_functions_to_xml(function_arguments)))
                     if image_message_list:
-                        self.conversation[convo_id].append({"role": "user", "content": [{"type": "text", "text": message}] + image_message_list})
+                        self.conversation[convo_id].append(UserMessage(message + image_message_list, Texts("\n\nYour message **must** end with [done] to signify the end of your output.", name="done", visible=self.check_done)))
                     else:
-                        self.conversation[convo_id].append({"role": "user", "content": message})
+                        self.conversation[convo_id].append(UserMessage(message, Texts("\n\nYour message **must** end with [done] to signify the end of your output.", name="done", visible=self.check_done)))
                 else:
-                    self.conversation[convo_id].append({"role": "assistant", "content": "我已经执行过这个工具了，接下来我需要做什么？"})
-
+                    self.conversation[convo_id].append(AssistantMessage("我已经执行过这个工具了，接下来我需要做什么？"))
         else:
             self.logger.error(f"error: add_to_conversation message is None or empty, role: {role}, function_name: {function_name}, message: {message}")
 
-        conversation_len = len(self.conversation[convo_id]) - 1
-        message_index = 0
         # if self.print_log:
         #     replaced_text = json.loads(re.sub(r';base64,([A-Za-z0-9+/=]+)', ';base64,***', json.dumps(self.conversation[convo_id])))
         #     self.logger.info(json.dumps(replaced_text, indent=4, ensure_ascii=False))
-        while message_index < conversation_len:
-            if self.conversation[convo_id][message_index]["role"] == self.conversation[convo_id][message_index + 1]["role"]:
-                if self.conversation[convo_id][message_index].get("content") and self.conversation[convo_id][message_index + 1].get("content") \
-                and self.conversation[convo_id][message_index].get("content") != self.conversation[convo_id][message_index + 1].get("content"):
-                    if type(self.conversation[convo_id][message_index + 1]["content"]) == str \
-                    and type(self.conversation[convo_id][message_index]["content"]) == list:
-                        self.conversation[convo_id][message_index + 1]["content"] = [{"type": "text", "text": self.conversation[convo_id][message_index + 1]["content"]}]
-                    if type(self.conversation[convo_id][message_index]["content"]) == str \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == list:
-                        self.conversation[convo_id][message_index]["content"] = [{"type": "text", "text": self.conversation[convo_id][message_index]["content"]}]
-                    if type(self.conversation[convo_id][message_index]["content"]) == dict \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == str:
-                        self.conversation[convo_id][message_index]["content"] = [self.conversation[convo_id][message_index]["content"]]
-                        self.conversation[convo_id][message_index + 1]["content"] = [{"type": "text", "text": self.conversation[convo_id][message_index + 1]["content"]}]
-                    if type(self.conversation[convo_id][message_index]["content"]) == dict \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == list:
-                        self.conversation[convo_id][message_index]["content"] = [self.conversation[convo_id][message_index]["content"]]
-                    if type(self.conversation[convo_id][message_index]["content"]) == dict \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == dict:
-                        self.conversation[convo_id][message_index]["content"] = [self.conversation[convo_id][message_index]["content"]]
-                        self.conversation[convo_id][message_index + 1]["content"] = [self.conversation[convo_id][message_index + 1]["content"]]
-                    if type(self.conversation[convo_id][message_index]["content"]) == list \
-                    and type(self.conversation[convo_id][message_index + 1]["content"]) == dict:
-                        self.conversation[convo_id][message_index + 1]["content"] = [self.conversation[convo_id][message_index + 1]["content"]]
-                    self.conversation[convo_id][message_index]["content"] += self.conversation[convo_id][message_index + 1]["content"]
-                self.conversation[convo_id].pop(message_index + 1)
-                conversation_len = conversation_len - 1
-            else:
-                message_index = message_index + 1
 
         history_len = len(self.conversation[convo_id])
 
@@ -278,27 +236,6 @@ class chatgpt(BaseLLM):
             else:
                 break
 
-    def get_latest_file_content(self) -> str:
-        """
-        获取最新文件内容
-        """
-        result = ""
-        if self.latest_file_content:
-            for file_path, content in self.latest_file_content.items():
-                result += (
-                    "<file>"
-                    f"<file_path>{file_path}</file_path>"
-                    f"<file_content>{content}</file_content>"
-                    "</file>\n\n"
-                )
-            if result:
-                result = (
-                    "<latest_file_content>"
-                    f"{result}"
-                    "</latest_file_content>"
-                )
-        return result
-
     async def get_post_body(
         self,
         prompt: str,
@@ -309,8 +246,6 @@ class chatgpt(BaseLLM):
         stream: bool = True,
         **kwargs,
     ):
-        self.conversation[convo_id][0] = {"role": "system","content": self.system_prompt + "\n\n" + self.get_latest_file_content()}
-
         # 构造 provider 信息
         provider = {
             "provider": "openai",
@@ -321,13 +256,19 @@ class chatgpt(BaseLLM):
             "image": True
         }
 
+        done_message = self.conversation[convo_id].provider("done")
+        if self.check_done and done_message:
+            done_message.visible = False
+            if self.conversation[convo_id][-1][-1].name == "done":
+                self.conversation[convo_id][-1][-1].visible = True
+
         # 构造请求数据
         request_data = {
             "model": model or self.engine,
-            "messages": copy.deepcopy(self.conversation[convo_id]) if pass_history else [
-                {"role": "system","content": self.system_prompt + "\n\n" + self.get_latest_file_content()},
-                {"role": role, "content": prompt}
-            ],
+            "messages": await self.conversation[convo_id].render_latest() if pass_history else Messages(
+                SystemMessage(self.system_prompt, self.conversation[convo_id].provider("files")),
+                UserMessage(prompt)
+            ),
             "stream": stream,
             "temperature": kwargs.get("temperature", self.temperature)
         }
@@ -643,8 +584,11 @@ class chatgpt(BaseLLM):
                         else:
                             yield chunk
                 if tool_name == "read_file" and "<tool_error>" not in tool_response:
-                    self.latest_file_content[tool_info['parameter']["file_path"]] = tool_response
+                    self.conversation[convo_id].provider("files").update(tool_info['parameter']["file_path"], tool_response)
                     all_responses.append(f"[{tool_name}({tool_args}) Result]:\n\nRead file successfully! The file content has been updated in the tag <latest_file_content>.")
+                elif tool_name == "get_knowledge_graph_tree" and "<tool_error>" not in tool_response:
+                    self.conversation[convo_id].provider("knowledge_graph").visible = True
+                    all_responses.append(f"[{tool_name}({tool_args}) Result]:\n\nGet knowledge graph tree successfully! The knowledge graph tree has been updated in the tag <knowledge_graph_tree>.")
                 elif tool_name == "write_to_file" and "<tool_error>" not in tool_response:
                     all_responses.append(f"[{tool_name} Result]:\n\n{tool_response}")
                 elif tool_name == "read_image" and "<tool_error>" not in tool_response:
@@ -656,9 +600,6 @@ class chatgpt(BaseLLM):
                 else:
                     all_responses.append(f"[{tool_name}({tool_args}) Result]:\n\n{tool_response}")
 
-            if self.check_done:
-                all_responses.append("Your message **must** end with [done] to signify the end of your output.")
-
             # 合并所有工具响应
             function_response = "\n\n".join(all_responses).strip()
             if missing_required_params:
@@ -744,7 +685,9 @@ class chatgpt(BaseLLM):
         need_done_prompt = False
 
         # 发送请求并处理响应
-        for i in range(10):
+        retry_times = 0
+        error_to_raise = None
+        while True:
             tmp_post_json = copy.deepcopy(json_post)
             if need_done_prompt:
                 tmp_post_json["messages"].extend(need_done_prompt)
@@ -797,6 +740,10 @@ class chatgpt(BaseLLM):
                             raise ModelNotFoundError(f"Model: {model or self.engine} not found!")
                         if "HTTP Error', 'status_code': 429" in processed_chunk:
                             raise RateLimitError(f"Rate limit exceeded for model: {model or self.engine}")
+                        if "HTTP Error', 'status_code': 413" in processed_chunk:
+                            raise InputTokenCountExceededError(processed_chunk)
+                        if "HTTP Error', 'status_code': 400" in processed_chunk:
+                            raise BadRequestError(f"Bad Request: {processed_chunk}")
                         if "HTTP Error', 'status_code': " in processed_chunk:
                             raise HTTPError(f"HTTP Error: {processed_chunk}")
                     yield processed_chunk
@@ -809,7 +756,7 @@ class chatgpt(BaseLLM):
                 return # Stop iteration
             except httpx.RemoteProtocolError:
                 continue
-            except httpx.ReadError:
+            except httpx.ReadError as e:
                 self.logger.warning(f"{e}, retrying...")
                 continue
             except APITimeoutError:
@@ -821,6 +768,16 @@ class chatgpt(BaseLLM):
             except RateLimitError as e:
                 self.logger.warning(f"{e}, retrying...")
                 continue
+            except InputTokenCountExceededError as e:
+                self.logger.error(f"The request body is too long: {e}")
+                error_to_raise = e
+                break
+            except BadRequestError as e:
+                self.logger.error(f"Bad request error: {e}")
+                raise
+            except GeneratorExit:
+                self.logger.warning("GeneratorExit caught, closing stream.")
+                break
             except ValidationError as e:
                 self.logger.warning(f"Validation failed: {e}. Retrying with corrective prompt.")
                 need_done_prompt = [
@@ -843,9 +800,13 @@ class chatgpt(BaseLLM):
                     error_message = "您输入了无效的API URL，请使用正确的URL并使用`/start`命令重新设置API URL。具体错误如下：\n\n" + str(e)
                     raise ConfigurationError(error_message)
                 # 最后一次重试失败，向上抛出异常
-                if i == 10:
+                retry_times += 1
+                if retry_times == 9:
                     raise RetryFailedError(str(e))
 
+        if error_to_raise:
+            raise error_to_raise
+
     def ask_stream(
         self,
         prompt: list,
@@ -966,9 +927,8 @@ class chatgpt(BaseLLM):
         Reset the conversation
         """
         self.system_prompt = system_prompt or self.system_prompt
-        self.latest_file_content = {}
-        self.conversation[convo_id] = [
-            {"role": "system", "content": self.system_prompt},
-        ]
+        self.conversation[convo_id] = Messages(
+            SystemMessage(Texts("system_prompt", self.system_prompt), self.conversation[convo_id].provider("files")),
+        )
         self.tokens_usage[convo_id] = 0
         self.current_tokens[convo_id] = 0
