@@ -19,6 +19,7 @@ from .screenshot import save_screenshot_to_file
 from .worker import worker, worker_gen, chatgroup
 from .click import find_and_click_element, scroll_screen
 from .subtasks import create_task, resume_task, get_all_tasks_status, get_task_result, create_tasks_from_csv
+from .deep_search import deepsearch
 
 #显式导入 aient.plugins 中的所需内容
 from ..aient.aient.plugins import (
@@ -46,6 +47,7 @@ __all__ = [
     "worker_gen",
     "read_image",
     "search_web",
+    "deepsearch",
     "create_task",
     "resume_task",
     "search_arxiv",
